/**
 * 
 */
package ch.ethz.se.rpnc.visitor;


import java.util.EmptyStackException;
import java.util.Stack;

import ch.ethz.se.rpnc.exception.IllegalNumberOfOperandsException;
import ch.ethz.se.rpnc.node.DivideByOperator;
import ch.ethz.se.rpnc.node.MinusOperator;
import ch.ethz.se.rpnc.node.Node;
import ch.ethz.se.rpnc.node.Operand;
import ch.ethz.se.rpnc.node.PlusOperator;
import ch.ethz.se.rpnc.node.TimesOperator;

/**
 * Evaluates expressions.
 */
public class Evaluator implements Visitor
{

	/**
	 * Last evaluation result.
	 */
	private double result;
	
	/**
	 * Stack to evaluate on.
	 */
	private Stack<Node> stack;

	/**
	 * Create new evaluator.
	 * @param s Stack to evaluate expressions on.
	 */
	public Evaluator (Stack<Node> s)
	{
		stack = s;
	}
	
	/**
	 * Last evaluation result.
	 * @return Last evaluation result.
	 */
	public double getResult ()
	{
		return result;
	}
	
	/**
	 * Visit the operand on the stack, update result.
	 */
	public void visit(Operand o)
	{
		result = o.getValue();
	}

	private double getNextOperand() throws Exception
	{
		Node n;
		try
		{
			n = stack.pop();
			n.accept(this);
			return result;
		}
		catch (EmptyStackException ese)
		{
			throw new IllegalNumberOfOperandsException ();
		}
	}
	
	/**
	 * Visit operator "+", update result
	 */
	public void visit(PlusOperator o) throws Exception
	{
		double d1;
		double d2;
		d1 = getNextOperand();
		d2= getNextOperand();
		result = d1 + d2;
	}

	/**
	 * Visit operator "-", update result
	 */
	public void visit(MinusOperator o) throws Exception
	{
		double d1;
		double d2;
		d1 = getNextOperand();
		d2= getNextOperand();
		result = d2 - d1;
	}

	/**
	 * Visit operator "*", update result
	 */
	public void visit(TimesOperator o) throws Exception
	{
		double d1;
		double d2;
		d1 = getNextOperand();
		d2= getNextOperand();
		result = d2 * d1;
	}

	/**
	 * Visit operator "/", update result
	 */
	public void visit(DivideByOperator o) throws Exception
	{
		double d1;
		double d2;
		d1 = getNextOperand();
		d2= getNextOperand();
		result = d2 / d1;
	}

}
